@@ -5,6 +5,7 @@ import platform
 import re
 
 from collections import defaultdict, UserDict
+from packaging import version as version_parse
 
 import pytablewriter
 
@@ -22,7 +23,7 @@ class Result:
         if self.exception:
             return f"Raised  ``{self.exception}`` Exception"
         elif not self.matched_expected:
-            return f"**Incorrect Result** (``{self.parsed_value}``)"
+            return "❌"
         else:
             return self.formatted_timing()
 
@@ -32,10 +33,10 @@ class ModuleResults(UserDict):
         non_exception_results = [(_python_version, result) for _python_version, result in self.data.items() if result.exception is None]
         return sorted(non_exception_results, key=lambda kvp: kvp[0], reverse=True)[0][1]
 
-FILENAME_REGEX_RAW = r"benchmark_timings_python(\d)(\d).csv"
+FILENAME_REGEX_RAW = r"benchmark_timings_python(\d)(\d\d?).csv"
 FILENAME_REGEX = re.compile(FILENAME_REGEX_RAW)
 
-MODULE_VERSION_FILENAME_REGEX_RAW = r"module_versions_python(\d)(\d).csv"
+MODULE_VERSION_FILENAME_REGEX_RAW = r"module_versions_python(\d)(\d\d?).csv"
 MODULE_VERSION_FILENAME_REGEX = re.compile(MODULE_VERSION_FILENAME_REGEX_RAW)
 
 UNITS = {"nsec": 1e-9, "usec": 1e-6, "msec": 1e-3, "sec": 1.0}
@@ -64,9 +65,11 @@ def format_used_module_versions(module_versions_used):
         if len(versions) == 1:
             results.append(f"{module}=={next(iter(versions.keys()))}")
         else:
-            results.append(", ".join([f"{module}=={version} (on Python {', '.join(sorted(py_versions))})" for version, py_versions in versions.items()]))
+            results.append(", ".join([f"{module}=={version} (on Python {', '.join(version_sort(py_versions))})" for version, py_versions in versions.items()]))
     return results
 
+def version_sort(versions):
+    return [str(v) for v in sorted([version_parse.parse(v) for v in versions])]
 
 def relative_slowdown(subject, comparison):
     most_modern_common_version = next(iter(sorted(set(subject.keys()).intersection(set(comparison)), reverse=True)), None)
@@ -92,8 +95,12 @@ def load_benchmarking_results(results_directory):
             with open(csv_file, "r") as fin:
                 reader = csv.reader(fin, delimiter=",", quotechar='"')
                 major, minor, timestamp = next(reader)
+                major = int(major)
+                minor = int(minor)
                 timestamps.add(timestamp)
                 for module, _setup, stmt, parse_result, count, time_taken, matched, exception in reader:
+                    if module == "hardcoded":
+                        continue
                     timing = float(time_taken) / int(count) if exception == "" else None
                     exception = exception if exception != "" else None
                     results[module][(major, minor)] = Result(
@@ -114,22 +121,28 @@ def load_benchmarking_results(results_directory):
     python_versions_by_modernity = sorted(python_versions, reverse=True)
     return results, python_versions_by_modernity, calling_code
 
+SPACER_COLUMN = ["…"]
 
 def write_benchmarking_results(results_directory, output_file, baseline_module, include_call):
     results, python_versions_by_modernity, calling_code = load_benchmarking_results(results_directory)
     modules_by_modern_speed = [module for module, results in sorted([*results.items()], key=lambda kvp: kvp[1].most_modern_result().timing)]
 
+    # GitHub in desktop browsers displays 830 pixels in a table width before adding a scroll bar.
+    # Experimentally, this means we can show the results from the 4 latest versions of Python and our important slowdown summary before it cuts off
+    # We add a spacer column before continuing with older versions of Python so that the slowdown summary isn't lost in the noise.
+    modern_versions_before_slowdown_summary = 4
+
     writer = pytablewriter.RstGridTableWriter()
-    formatted_python_versions = ["Python {}".format(".".join(key)) for key in python_versions_by_modernity]
-    writer.header_list = ["Module"] + (["Call"] if include_call else []) + formatted_python_versions + [f"Relative Slowdown (versus {baseline_module}, latest Python)"]
-    writer.type_hint_list = [pytablewriter.String] * len(writer.header_list)
+    formatted_python_versions = [f"Python {major}.{minor}" for major, minor in python_versions_by_modernity]
+    writer.headers = ["Module"] + (["Call"] if include_call else []) + formatted_python_versions[0:modern_versions_before_slowdown_summary] + [f"Relative slowdown (versus {baseline_module}, latest Python)"] + SPACER_COLUMN + formatted_python_versions[modern_versions_before_slowdown_summary:]
+    writer.type_hints = [pytablewriter.String] * len(writer.headers)
 
     calling_codes = [calling_code[module] for module in modules_by_modern_speed]
     performance_results = [[results[module].get(python_version, NOT_APPLICABLE) for python_version in python_versions_by_modernity] for module in modules_by_modern_speed]
     relative_slowdowns = [relative_slowdown(results[module], results[baseline_module]) if module != baseline_module else NOT_APPLICABLE for module in modules_by_modern_speed]
 
     writer.value_matrix = [
-        [module] + ([calling_code[module]] if include_call else []) + performance_by_version + [relative_slowdown] for module, calling_code, performance_by_version, relative_slowdown in zip(modules_by_modern_speed, calling_codes, performance_results, relative_slowdowns)
+        [module] + ([calling_code[module]] if include_call else []) + performance_by_version[0:modern_versions_before_slowdown_summary] + [relative_slowdown] + SPACER_COLUMN + performance_by_version[modern_versions_before_slowdown_summary:] for module, calling_code, performance_by_version, relative_slowdown in zip(modules_by_modern_speed, calling_codes, performance_results, relative_slowdowns)
     ]
 
     with open(output_file, "w") as fout:
@@ -137,15 +150,16 @@ def write_benchmarking_results(results_directory, output_file, baseline_module,
         writer.write_table()
         fout.write("\n")
 
-        if len(modules_by_modern_speed) > 1:
+        latest_python_version = python_versions_by_modernity[0]
+        modules_supporting_latest_python = [module for module in modules_by_modern_speed if latest_python_version in results[module]]
+        if len(modules_supporting_latest_python) > 1:
             baseline_module_timing = results[baseline_module].most_modern_result().formatted_timing()
 
-            fastest_module, next_fastest_module = modules_by_modern_speed[0:2]
+            fastest_module, next_fastest_module = modules_supporting_latest_python[0:2]
             if fastest_module == baseline_module:
-                fout.write(f"{baseline_module} takes {baseline_module_timing}, which is **{relative_slowdown(results[next_fastest_module], results[baseline_module])} faster than {next_fastest_module}**, the next fastest ISO 8601 parser in this comparison.\n")
+                fout.write(f"{baseline_module} takes {baseline_module_timing}, which is **{relative_slowdown(results[next_fastest_module], results[baseline_module])} faster than {next_fastest_module}**, the next fastest {formatted_python_versions[0]} parser in this comparison.\n")
             else:
-                fout.write(f"{baseline_module} takes {baseline_module_timing}, which is **{relative_slowdown(results[baseline_module], results[fastest_module])} slower than {fastest_module}**, the fastest ISO 8601 parser in this comparison.\n")
-
+                fout.write(f"{baseline_module} takes {baseline_module_timing}, which is **{relative_slowdown(results[baseline_module], results[fastest_module])} slower than {fastest_module}**, the fastest {formatted_python_versions[0]} parser in this comparison.\n")
 
 def load_module_version_info(results_directory):
     module_versions_used = defaultdict(dict)
